/* Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.blueland.bpm.engine.core.cmd;

import java.io.Serializable;
import java.util.Map;

import org.activiti.engine.impl.interceptor.Command;
import org.activiti.engine.impl.interceptor.CommandContext;
import org.apache.commons.lang3.StringUtils;

import com.blueland.bpm.engine.core.model.ExceptionErrorCode;
import com.blueland.bpm.engine.core.service.ProcessTransitionFreeService;
import com.blueland.bpm.engine.util.WorkFlowException;

/**
 * 退回上一步命令类
 * @author lw
 */
public class CallBackPrevTaskCmd implements Command<Void>, Serializable {

	private static final long serialVersionUID = 1L;

	protected String taskId;
	protected String backActivityId;
	protected Map<String,Object> variables;

	public CallBackPrevTaskCmd(String taskId,String backActivityId,Map<String,Object> variables) {
		this.taskId = taskId;
		this.backActivityId=backActivityId;
		this.variables=variables;
	}

	public Void execute(CommandContext commandContext) {
		if (StringUtils.isEmpty(taskId)) {
			throw new WorkFlowException(ExceptionErrorCode.B2001,"taskId is empty");
		}
		if (StringUtils.isEmpty(backActivityId)) {
			throw new WorkFlowException(ExceptionErrorCode.B2001,"backActivityId is empty");
		}
		ProcessTransitionFreeService baseWfService = new ProcessTransitionFreeService(
				commandContext.getProcessEngineConfiguration());
		baseWfService.commitProcess(taskId,backActivityId, variables);
		return null;
	}

}
