/* Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.blueland.bpm.engine.core.cmd;

import java.io.Serializable;
import java.util.Map;

import org.activiti.engine.impl.interceptor.Command;
import org.activiti.engine.impl.interceptor.CommandContext;
import org.activiti.engine.impl.persistence.entity.HistoricProcessInstanceEntity;
import org.activiti.engine.impl.persistence.entity.TaskEntity;
import org.apache.commons.lang3.StringUtils;

import com.blueland.bpm.engine.core.model.ExceptionErrorCode;
import com.blueland.bpm.engine.core.model.ProcessInputModel;
import com.blueland.bpm.engine.core.service.ProcessTransitionFreeService;
import com.blueland.bpm.engine.util.ProcessDefinitionUtils;
import com.blueland.bpm.engine.util.WorkFlowContants;
import com.blueland.bpm.engine.util.WorkFlowException;

/**
 * 退回起草命令类
 * @author lw
 */
public class CallBackToStartActivityCmd implements Command<Void>, Serializable {

	private static final long serialVersionUID = 1L;
	protected String taskId;
	protected String userId;
	protected Map<String,Object> variables;

	public CallBackToStartActivityCmd(String taskId,String userId,Map<String,Object> variables) {
		this.taskId = taskId;
		this.userId=userId;
		this.variables=variables;
	}

	public Void execute(CommandContext commandContext) {
		if (StringUtils.isEmpty(taskId)) {
			throw new WorkFlowException(ExceptionErrorCode.B2001,"taskId is empty");
		}
		if (StringUtils.isEmpty(userId)) {
			throw new WorkFlowException(ExceptionErrorCode.B2001,"userId is empty");
		}
		TaskEntity taskEntity = commandContext.getTaskEntityManager()
				.findTaskById(taskId);
		HistoricProcessInstanceEntity historicProcessInstanceEntity = commandContext
				.getHistoricProcessInstanceEntityManager()
				.findHistoricProcessInstance(taskEntity.getProcessInstanceId());
		ProcessInputModel processInputModel = ProcessDefinitionUtils
				.getWfprocessInputModel(variables);
		processInputModel.setWf_nextActDefId(historicProcessInstanceEntity
				.getStartActivityId());
		processInputModel.setWf_receiver(historicProcessInstanceEntity
				.getStartUserId());
		processInputModel.setWf_sender(userId);
		processInputModel.setWf_procInstId(historicProcessInstanceEntity
				.getProcessInstanceId());
		processInputModel.setWf_curActDefId(taskEntity.getTaskDefinitionKey());
		processInputModel.setWf_curActDefName(taskEntity.getName());
		processInputModel.setWf_curActInstId(taskEntity.getId());
		if(StringUtils.isEmpty(processInputModel.getWf_procTitle())){
			processInputModel.setWf_procTitle(taskEntity.getProcTitle());
		}
		variables.put(WorkFlowContants.WF_PROCESS_INPUT_VARIABLE_KEY,
				processInputModel);
		taskEntity.setActionType(processInputModel.getWf_actionType());
		ProcessTransitionFreeService baseWfService = new ProcessTransitionFreeService(
				commandContext.getProcessEngineConfiguration());
		baseWfService.commitProcess(taskEntity.getId(),
				historicProcessInstanceEntity.getStartActivityId(), variables);
		return null;
	}

}
